

class Provider:
    OPENAI = "openai"
    ANTHROPIC = "Anthropic"
    AWS_BEDROCK = "awsbedrock"
    AZURE_OPENAI = "azure"
    CEREBRAS = "cerebras"
    COHERE = "cohere"
    GEMINI = "gemini"
    VERTEX_AI = "vertex_ai"
    GROQ = "groq"
    NVIDIA = "nvidia"
    SAMBANOVA = "sambanova"

